library(ape)
library(HierDpart)
library(genomalicious)
library(ggtree)

FUN_top_loading_loci <- function(E, prop){
  # E = matrix, the 1:m eigen vectors to use.
  # prop = numeric, proportion of top loci to keep based on absolute loading
  num_loci <- nrow(E)
  
  lapply(1:ncol(E), function(i){
    e_vec <- E[,i]
    
    e_top <- rev(sort(abs(e_vec)))[1:round(num_loci*prop,0)]
    
    data.table(VECTOR=i, LOCUS=names(e_top), LOAD.ABS=e_top)
  }) %>% 
    do.call('rbind',.) %>% 
    return()
}

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   DATA   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

out_dir <- '01_04_Genetic_analysis'
if(dir.exists(out_dir)==FALSE){ dir.create(out_dir)}

### THE SNP DATA
# snpData <- vcf2DT(vcfFile = '01_03_Variant_calling/vars_bi_minq_mindp_miss_snp.vcf') %>%
#   as.data.table
# 
# snpData[, POP:=SAMPLE]

# saveRDS(snpData, paste0(out_dir,'/snp_data_table.RDS'))
snpData <- readRDS(paste0(out_dir,'/snp_data_table.RDS')) 


# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   MISSING DATA AMONG SAMPLES   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

snpData %>% 
  .[, sum(GT=='./.')/length(GT), by=SAMPLE]

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   STEP 1: BIALLELIC LOCI   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

uniqAlleles <- snpData[, c('LOCUS','REF','ALT')] %>% unique

step_1 <- uniqAlleles[!LOCUS %in% uniqAlleles[grep(',',ALT)]$LOCUS] %>% 
  .[['LOCUS']]

snpData[LOCUS %in% step_1, unique(GT)]

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   STEP 2: MINOR ALLLELE FREQUENCE   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

maf <- 2/14

step_2 <- filter_maf(snpData[LOCUS %in% step_1], maf=maf, method='mean')

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   STEP 3: SUBSET TO JUST THE CHROMOSOMES   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

step_3 <- snpData %>% 
  .[LOCUS %in% step_2] %>% 
  .[CHROM %in% c('CM037069.1','CM037070.1','CM037071.1','CM037072.1')] %>% 
  .[['LOCUS']] %>% 
  unique

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   STEP 4: EVENLY SPACE LOCI   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

step_4 <- filter_space_loci(snpData[LOCUS %in% step_3], stepSize=5000)

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   WORKING DATASET   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

### WORKING SNP SET
snpWork <- snpData[LOCUS %in% step_4] %>% 
  .[ , GT:=genoscore_converter(GT)]

# saveRDS(snpWork, paste0(out_dir, '/snp_working_table.RDS'))
snpWork <- readRDS(paste0(out_dir, '/snp_working_table.RDS'))

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   GENETIC DISTANCE   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
genoMat <- snpWork %>% 
  DT2Mat_genos() %>% 
  t()

uniq_samps <- snpWork$SAMPLE %>% unique

sampCombos <- CJ(POP1=uniq_samps, POP2=uniq_samps)

ddLong <- lapply(1:nrow(sampCombos), function(i){
  pop1 <- sampCombos$POP1[i]
  pop2 <- sampCombos$POP2[i]
  struc <- Str(nreg=1, r=2, n=2)
  abun <- genoMat[,c(pop1,pop2)]
  IDIP(abun, struc)['Differentiation.1',] %>% 
    data.table(POP1=pop1,POP2=pop2,DD=.)
}) %>% 
  do.call('rbind',.)

ddLong[POP1!=POP2] %>% 
  ggplot(., aes(x=DD)) +
  theme_classic() +
  geom_histogram(colour='grey20', fill='grey80') +
  labs(x='Genetic distance (deltaD)')

ddDist <- pairwiseMat2DT(dat=ddLong, flip=TRUE, X1='POP1', X2='POP2', Y='DD', diagAdd=FALSE)

ddLong[POP1=='OAT_02'] %>% 
  copy %>% 
  setorder(., -DD) %>% 
  print

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   DESIGNATE CLUSTERS   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

Klist <- list(
  data.table(CLUSTER='1', SAMPLE=paste0('OAT_', c('16','05','02'))),
  data.table(CLUSTER='2', SAMPLE=paste0('OAT_', c('21','30','01','09','04','23','22'))),
  data.table(CLUSTER='3', SAMPLE=paste0('OAT_', c('28','29','26','03')))
)

Ktable <- Klist %>% do.call('rbind',.) %>% 
  .[, SAMPLE_CLUSTER:=paste0(SAMPLE, '_', CLUSTER)]

# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   NEIGHBOUR JOINING TREE   ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

ddNJ <- ape::nj(as.dist(ddDist))

plot(ddNJ)

ddNJtree <- ggtree(ddNJ)
ddNJtree$data <- left_join(ddNJtree$data, Ktable, by=c('label'='SAMPLE'))

plot(ddNJtree) +
  geom_tiplab(mapping=aes(colour=CLUSTER)) +
  xlim(0, 0.0014)

samp_clust_levels <- ddNJtree$data %>% 
  as.data.table %>% 
  .[isTip==TRUE] %>% 
  copy %>% 
  setorder(., CLUSTER, y) %>% 
  .[, paste0(label, '_', CLUSTER)]


# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
####   PRINCIPAL COMPONENTS ANALYSIS  ####
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

PCA <- snpWork %>%
  left_join(., Ktable) %>% 
  pca_genos(., scaling='covar', popCol='CLUSTER')
  
PCA$exp_var <- round((PCA$sdev^2)/sum(PCA$sdev^2)*100)

topLoci <- FUN_top_loading_loci(PCA$rotation[,1:3], 0.01)

PCA$tab <- PCA$x %>% 
  as.data.frame %>% 
  rownames_to_column(., 'SAMPLE') %>% 
  left_join(., Ktable) %>% 
  as.data.table()

ggplot(PCA$tab, aes(x=PC1, y=PC2, colour=CLUSTER)) +
  theme_classic() +
  geom_hline(yintercept = 0, linetype='longdash')+
  geom_vline(xintercept = 0, linetype='longdash')+
  geom_point(size=3) +
  labs(
    x=paste0('PC1: ', PCA$exp_var[1], '%'),
    y=paste0('PC2: ', PCA$exp_var[2], '%')
  )

ggplot(PCA$tab, aes(x=PC3, y=PC4, colour=CLUSTER)) +
  theme_classic() +
  geom_hline(yintercept = 0, linetype='longdash')+
  geom_vline(xintercept = 0, linetype='longdash')+
  geom_point(size=3) +
  labs(
    x=paste0('PC3: ', PCA$exp_var[3], '%'),
    y=paste0('PC4: ', PCA$exp_var[4], '%'),
    colour='Cluster'
  )

### HAPLOTYPTE MAP
snpWork %>% 
  .[LOCUS %in% unique(topLoci$LOCUS)] %>% 
  left_join(., Ktable) %>% 
  as.data.table %>% 
  .[, SAMPLE_CLUSTER:=factor(SAMPLE_CLUSTER, levels=samp_clust_levels)] %>% 
  ggplot(., aes(x=LOCUS, y=SAMPLE_CLUSTER, fill=as.factor(GT))) +
  theme(
    legend.position = 'top',
    axis.text.x=element_blank(),
    axis.ticks.x=element_blank(),
    panel.background = element_blank()) +
  geom_tile()+
  scale_fill_manual(values=c('0'='#34C4FF','1'='#381FEA','2'='#D8004F')) +
  labs(x='Locus', y='Sample', fill='Genotype')

